<?php
/* Copyright (c) 2014 Synology Inc. All rights reserved. */

class SynoGoFileMe extends abstractSynoFileHosting {

	private $MAINURL = 'https://gofile.me/RequestTL.php';
	private $CookiePath;
	private $CookiePathPrefix = "/var/services/download";
	private $CookieFolderName = "synodl_cookie_folder";
	private $CookieFileName = "synodl_cookie";

	private function TestDownloadLink($link) {
		$browser = new SynoBrowser();
		$browser->setNoBody(true);
		$browser->setCookieFile($this->CookiePath);
		$browser->browseRetry($link, 5, array(200, 302, 404));
		$httpRetCode = $browser->getHttpStatus();
		switch($httpRetCode) {
			case 200:
			case 302:
				return true;
			case 404:
				return ERR_FILE_NO_EXIST;
			default:
				syslog(LOG_ERR, "Failed to test download link $link ($httpRetCode)");
				return ERR_UNKNOWN;
		}
	}

	private function BrowseForCookie($link) {
		$browser = new SynoBrowser();
		$browser->setCookieFile($this->CookiePath);
		$browser->browseRetry($link, 5);
		$httpRetCode = $browser->getHttpStatus();
		if (200 != $httpRetCode) {
			syslog(LOG_ERR, "Failed to browse site $link ($httpRetCode)");
			return false;
		}
		return true;
	}

	private function UpdateCookiePathByCorrespondingTask() {
		GLOBAL $DownloadParams;
		$cookieFolder = $this->CookiePathPrefix . "/" . $DownloadParams[PARAMS_TASK_ID] . "/" . $this->CookieFolderName;
		if (!file_exists($cookieFolder)) {
			mkdir($cookieFolder, 0777);
		}
		$this->CookiePath = $cookieFolder . "/" . $this->CookieFileName;
	}

	private function GetTunnelInfo($serverID, $sharingID) {
		$formData = array(
			'server_ID' => $serverID,
			'sharing_ID' => $sharingID
		);
		$browser = new SynoBrowser();
		$browser->setPost($formData);
		$browser->setCookieFile($this->CookiePath);
		if (200 != $browser->browse($this->MAINURL)) {
			return false;
		}
		if (null == $retData = json_decode($browser, True)) {
			return false;
		}
		return $retData;
	}

	public function GetDownloadInfo() {
		$DownloadData = $this->createData("DownloadData");
		$downloadurl = "";

		$this->UpdateCookiePathByCorrespondingTask();

		if (preg_match('/\/sharing\/([\w]{9})$/', $this->Url, $matches)) {
			if ($this->BrowseForCookie($this->Url)) {
				$downloadurl = str_replace("sharing", "fsdownload", $this->Url)."/share";
			} else {
				$DownloadData->error = ERR_UNKNOWN;
			}
			goto End;
		}
		if (!preg_match('/http[s]?:\/\/gofile.me\/(\w*)\/(\w*)/', $this->Url, $matches)) {
			$DownloadData->error = ERR_UNKNOWN;
			goto End;
		}

		$maxRetry = 5;
		$retry = 0;
		while ($retry < $maxRetry && false == ($retData = $this->GetTunnelInfo($matches[1], $matches[2]))) {
			$retry++;
			$tmp = floor(pow(2, $retry - 1) * 1000000 * rand(70, 130)/100);
			usleep($tmp);
		}
		if (false == $retData) {
			syslog(LOG_ERR, "failed to create tunnel");
			$DownloadData->error = ERR_UNKNOWN;
			goto End;
		}

		$host = $retData['service']['relay_dn'];
		$port = $retData['service']['relay_port'];
		$hash = $matches[2];
		$DSMVersion = $retData['server']['version'];
		$isDSM6orLater = (7135 <= intval($DSMVersion));
		if (!$isDSM6orLater) {
			/* direct download */
			$downloadurl = "https://$host/tunnel/$port/fbsharing/$hash";
			goto End;
		}

		$downloadLink = "https://$host/tunnel/$port/sharing/$hash";
		if (!$this->BrowseForCookie($downloadLink)) {
			$DownloadData->error = ERR_UNKNOWN;
			goto End;
		}
		/* Give a fake filename - "share" for downloading,
		* real filename will be given in content-disposition */
		$downloadurl = "https://$host/tunnel/$port/fsdownload/$hash/share";
End:
		if (!empty($downloadurl)) {
			$result = $this->TestDownloadLink($downloadurl);
			if (true !== $result) {
				$DownloadData->error = $result;
			} else {
				$DownloadData->downloadurl = $downloadurl;
				$DownloadData->cookiepath = $this->CookiePath;
				$DownloadData->multiconnection = false;
			}
		}
		return $DownloadData;
	}
}
?>
