# -*- coding: utf-8 -*-

from ..internal.misc import json
from ..internal.MultiAccount import MultiAccount


class LinksnappyCom(MultiAccount):
    __name__ = "LinksnappyCom"
    __type__ = "account"
    """SYNO"""
    __version__ = "0.18"
    """SYNO END"""
    __status__ = "testing"

    __config__ = [("mh_mode", "all;listed;unlisted", "Filter hosters to use", "all"),
                  ("mh_list", "str", "Hoster list (comma separated)", ""),
                  ("mh_interval", "int", "Reload interval in hours", 12)]

    __description__ = """Linksnappy.com account plugin"""
    __license__ = "GPLv3"
    """SYNO"""
    __authors__ = [("stickell", "l.stickell@yahoo.it"),
                   ("GammaC0de", "nitzo2001[AT]yahoo[DOT]com"),
                   ("Synology PAT", "pat@synology.com")]
    """SYNO END"""

    API_URL = "https://linksnappy.com/api/"

    def api_response(self, method, **kwargs):
        return json.loads(self.load(self.API_URL + method,
                                    get=kwargs))

    def grab_hosters(self, user, password, data):
        json_data = self.api_response("FILEHOSTS")
        """SYNO"""
        hosters = json_data['return'].keys()
        self.patch_hosters_for_uploaded_net(hosters)
        return hosters
        """SYNO END"""

    def grab_info(self, user, password, data):
        premium = True
        validuntil = None
        trafficleft = None

        json_data = self.api_response("USERDETAILS", username=user, password=password)

        if json_data['status'] != "OK":
            self.log_error(json_data['error'])

        else:
            validuntil = json_data['return']['expire']

            if validuntil == "lifetime":
                validuntil = -1

            elif validuntil == "expired":
                premium = False

            else:
                validuntil = float(validuntil)

            if 'trafficleft' not in json_data['return'] or isinstance(json_data['return']['trafficleft'], basestring):
                trafficleft = -1

            else:
                # @TODO: Remove `/ 1024` in 0.4.10
                trafficleft = float(json_data['return']['trafficleft']) / 1024

        return {'premium': premium,
                'validuntil': validuntil,
                'trafficleft': trafficleft,
                'options'    : {'limitDL': ['1']}}

    def signin(self, user, password, data):


        json_data = self.api_response("AUTHENTICATE", username=user, password=password)

        if json_data['status'] != "OK":
            self.fail_login(json_data['error'])


    """SYNO"""
    def patch_hosters_for_uploaded_net(self, hosters):
        if u'uploaded.net' in hosters:
            hosters.extend([u'uploaded.to', u'ul.to'])
    """SYNO END"""