# -*- coding: utf-8 -*-
import socket
import os
import struct
import json
import time

from module.plugins.internal.Addon import Addon
from module.plugins.internal.misc import threaded
from module.Api import Api

class DomainSocket(Addon):
    __name__ = "DomainSocket"
    __type__ = "hook"
    __version__ = "0.01"

    __config__ = [("activated", "bool", "Activated", True)]

    __description__ = """Synology domain socket hook plugin"""
    __license__ = ""
    __authors__ = [("Synology PAT", "pat@synology.com")]

    def coreReady(self):
        self.log_info(_("in coreReady"));
        self.server();

    @threaded
    def server(self):
        socketname = self.pyload.synoDomainsocketPath
        try:
            os.unlink(socketname)
        except OSError:
            pass
        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        os.umask(0077)
        sock.bind(socketname);
        os.umask(0000)
        sock.listen(5);

        while True:
            connection, client_address = sock.accept()
            try:
                #read header
                data = connection.recv(4);
                if len(data) != 4:
                    raise Exception("Wrong header size");
                size = struct.unpack('I', data)[0]
                #read body
                data = connection.recv(size)
                if len(data) != size:
                    raise Exception("Wrong body size");
                #call responding api
                ret = self.callApi(data)
                if ret:
                    strRet = json.dumps(ret)
                else:
                    strRet = json.dumps(True)
                lenStrRet = len(strRet)
                raw = struct.pack('I', lenStrRet)
                connection.sendall(raw)
                connection.sendall(strRet)
            except Exception as err:
                self.log_info(err)
            finally:
                connection.close()
    def callApi(self, request):
        data = json.loads(request)
        api = data["api"]
        args = data["args"]
        argsArray = []
        for arg in args:
            argsArray.append(arg)
        ret = getattr(Api, api)(self.pyload.api, *argsArray)
        return ret
