# -*- coding: utf-8 -*-

import re

from module.network.RequestFactory import getURL as get_url

from ..internal.Base import parse_fileInfo
from ..internal.SimpleHoster import SimpleHoster


def get_info(urls):
    for url in urls:
        h = get_url(url, just_header=True)
        m = re.search(r'Location: (.+)\r\n', h)

        if m and not re.match(
                m.group(1), FilefactoryCom.__pattern__):  #: It's a direct link! Skipping
            yield (url, 0, 7, url)
        else:
            #: It's a standard html page
            yield parse_fileInfo(FilefactoryCom, url, get_url(url))


class FilefactoryCom(SimpleHoster):
    __name__ = "FilefactoryCom"
    __type__ = "hoster"
    __version__ = "0.64"
    __status__ = "testing"

    """SYNO"""
    __pattern__ = r'https?://(?:www\.)?filefactory\.com/(file|trafficshare/\w+|search/get/\w+)/\w+'
    """SYNO END"""
    __config__ = [("activated", "bool", "Activated", True),
                  ("use_premium", "bool", "Use premium account if available", True),
                  ("fallback", "bool",
                   "Fallback to free download if premium fails", True),
                  ("chk_filesize", "bool", "Check file size", True),
                  ("max_wait", "int", "Reconnect if waiting time is greater than minutes", 10)]

    __description__ = """Filefactory.com hoster plugin"""
    __license__ = "GPLv3"
    """SYNO"""
    __authors__ = [("stickell", "l.stickell@yahoo.it"),
                   ("Walter Purcaro", "vuolter@gmail.com"),
                   ("Synology PAT", "pat@synology.com")]
    """SYNO END"""

    INFO_PATTERN = r'<div id="file_name"[^>]*>\s*<h2>(?P<N>[^<]+)</h2>\s*<div id="file_info">\s*(?P<S>[\d.,]+) (?P<U>[\w^_]+) uploaded'
    OFFLINE_PATTERN = r'<h2>File Removed</h2>|This file is no longer available|Invalid Download Link'

    """SYNO"""
    DIRECT_LINK = None
    """SYNO END"""

    LINK_FREE_PATTERN = LINK_PREMIUM_PATTERN = r'"([^"]+filefactory\.com/get.+?)"'

    WAIT_PATTERN = r'<div id="countdown_clock" data-delay="(\d+)">'
    PREMIUM_ONLY_PATTERN = r'>Premium Account Required'

    COOKIES = [("filefactory.com", "locale", "en_US.utf8")]


    """SYNO"""
    def handle_direct(self, pyfile):
        super(FilefactoryCom, self).handle_direct(pyfile)
        if self.link:
            self.empty_filename = True


    """
    Copy from Hoster.py. For action when http code is 302. Corresponding to pyload issue #2342.
    """
    def isresource(self, url, redirect=True, resumable=None):
        resource  = False
        maxredirs = 5

        if resumable is None:
            resumable = self.resume_download

        if type(redirect) is int:
            maxredirs = max(redirect, 1)

        elif redirect:
            maxredirs = int(self.pyload.api.getConfigValue("UserAgentSwitcher", "maxredirs", "plugin")) or maxredirs  #@TODO: Remove `int` in 0.4.10

        header = self.load(url, just_header=True)

        for i in xrange(1, maxredirs):
            if not redirect or header.get('connection') == "close":
                resumable = False

            if 'content-disposition' in header:
                resource = url

            elif header.get('location'):
                location = self.fixurl(header.get('location'), url)
                code     = header.get('code')

                if code == 302:
                    self.log_debug("Redirect #%d to: %s" % (i, location))
                    header = self.load(location, just_header=True)
                    url = location
                    continue

                elif code == 301 or resumable:
                    self.log_debug("Redirect #%d to: %s" % (i, location))
                    header = self.load(location, just_header=True)
                    url = location
                    continue

            else:
                from os.path import splitext
                from module.plugins.internal.misc import parse_name

                contenttype = header.get('content-type')
                extension   = splitext(parse_name(url))[-1]

                if contenttype:
                    mimetype = contenttype.split(';')[0].strip()

                elif extension:
                    mimetype = mimetypes.guess_type(extension, False)[0] or \
                               "application/octet-stream"

                else:
                    mimetype = None

                if mimetype and (resource or 'html' not in mimetype):
                    resource = url
                else:
                    resource = False

            return resource
    """SYNO END"""


    def handle_free(self, pyfile):
        if "Currently only Premium Members can download files larger than" in self.data:
            self.fail(_("File too large for free download"))
        elif "All free download slots on this server are currently in use" in self.data:
            self.retry(50, 15 * 60, _("All free slots are busy"))

        m = re.search(self.LINK_FREE_PATTERN, self.data)
        if m is None:
            return

        self.link = m.group(1)

        m = re.search(self.WAIT_PATTERN, self.data)
        if m is not None:
            self.wait(m.group(1))

    def check_download(self):
        check = self.scan_download({
            'multiple': "You are currently downloading too many files at once.",
            'error': '<div id="errorMessage">'
        })

        if check == "multiple":
            self.log_debug("Parallel downloads detected; waiting 15 minutes")
            self.retry(wait=15 * 60, msg=_("Parallel downloads"))

        elif check == "error":
            self.error(_("Unknown error"))

        return SimpleHoster.check_download(self)
