# -*- coding: utf-8 -*-

import re
import urlparse
import urllib

from ..internal.misc import json, decode, encode
from ..internal.MultiHoster import MultiHoster


class LinksnappyCom(MultiHoster):
    __name__ = "LinksnappyCom"
    __type__ = "hoster"
    __version__ = "0.18"
    __status__ = "testing"

    __pattern__ = r'https?://(?:[^/]+\.)?linksnappy\.com'
    __config__ = [("activated", "bool", "Activated", True),
                  ("use_premium", "bool", "Use premium account if available", True),
                  ("fallback", "bool", "Fallback to free download if premium fails", False),
                  ("chk_filesize", "bool", "Check file size", True),
                  ("max_wait", "int", "Reconnect if waiting time is greater than minutes", 10),
                  ("revert_failed", "bool", "Revert to standard download if fails", True)]

    __description__ = """Linksnappy.com multi-hoster plugin"""
    __license__ = "GPLv3"
    """SYNO"""
    __authors__ = [("stickell", "l.stickell@yahoo.it"),
                   ("Bilal Ghouri", None),
                   ("Synology PAT", "pat@synology.com")]
    """SYNO END"""

    def handle_premium(self, pyfile):
        host = self._get_host(pyfile.url)
        json_params = json.dumps({'link': pyfile.url,
                                  'type': host,
                                  'username': self.account.user,
                                  'password': self.account.get_login('password')})

        r = self.load("https://linksnappy.com/api/linkgen",
                      post={'genLinks': json_params})

        self.log_debug("JSON data: " + r)

        """SYNO"""
        try:
            res_json = json.loads(r)
        except TypeError:
            self.error(_("Error response"))

        j = res_json['links'][0]
        self.check_final_response(j['error'])
        """SYNO END"""

        """SYNO"""
        pyfile.name = decode(urllib.unquote(encode(j['filename'])))
        """SYNO END"""
        self.link = j['generated']

    @staticmethod
    def _get_host(url):
        host = urlparse.urlsplit(url).netloc
        """SYNO"""
        hoster = re.search(r'[\w\-]+\.\w+$', host).group(0)
        hoster = LinksnappyCom.patch_hoster_for_uploaded_net(hoster)
        return hoster
        """SYNO END"""

    """SYNO"""
    def check_final_response(self, error_msg):
        if error_msg:
            original_pattern = self.ERROR_PATTERN

            self.ERROR_PATTERN = r'(.*)'
            self.data = error_msg
            self.check_errors()

            self.ERROR_PATTERN = original_pattern

    @staticmethod
    def patch_hoster_for_uploaded_net(hoster):
        if 'uploaded.to' == hoster or 'ul.to' == hoster:
            return 'uploaded.net'
        else:
            return hoster
    """SYNO END"""
