# -*- coding: utf-8 -*-

"""SYNO"""
import re
import pycurl
"""SYNO END"""

"""SYNO"""
from module.plugins.internal.SimpleHoster import SimpleHoster
"""SYNO END"""


class TurbobitNet(SimpleHoster):
    __name__ = "TurbobitNet"
    __type__ = "hoster"
    __version__ = "0.31"
    """SYNO"""
    __status__ = "testing"
    """SYNO END"""

    __pattern__ = r'http://(?:www\.)?turbobit\.net/(?:download/free/)?(?P<ID>\w+)'
    __config__ = [("activated", "bool", "Activated", True),
                  ("use_premium", "bool", "Use premium account if available", True),
                  ("fallback", "bool",
                   "Fallback to free download if premium fails", True),
                  ("chk_filesize", "bool", "Check file size", True),
                  ("max_wait", "int", "Reconnect if waiting time is greater than minutes", 10)]

    __description__ = """Turbobit.net hoster plugin"""
    __license__ = "GPLv3"
    """SYNO"""
    __authors__ = [("zoidberg", "zoidberg@mujmail.cz"),
                   ("prOq", None),
                   ("Synology PAT", "pat@synology.com")]
    """SYNO END"""

    URL_REPLACEMENTS = [
        (__pattern__ + ".*", "http://turbobit.net/\g<ID>.html")]

    COOKIES = [("turbobit.net", "user_lang", "en")]

    NAME_PATTERN = r'id="file-title">(?P<N>.+?)<'
    SIZE_PATTERN = r'class="file-size">(?P<S>[\d.,]+) (?P<U>[\w^_]+)'
    OFFLINE_PATTERN = r'<h2>File Not Found</h2>|html\(\'File (?:was )?not found'

    """SYNO"""
    LINK_DOMAIN = 'http://turbobit.net'
    LINK_DOWNLOAD_FREE = LINK_DOMAIN + '/download/free/'
    LINK_DOWNLOAD_GET_LINK = LINK_DOMAIN + '/download/getLinkTimeout/'
    """SYNO END"""
    LINK_FREE_PATTERN = LINK_PREMIUM_PATTERN = r'(/download/redirect/[^"\']+)'

    LIMIT_WAIT_PATTERN = r'<div id=\'timeout\'>(\d+)<'
    CAPTCHA_PATTERN = r'<img alt="Captcha" src="(.+?)"'

    """SYNO"""
    @property
    def basic_download_free_url(self):
        return "%s%s" % (self.LINK_DOWNLOAD_FREE, self.info['pattern']['ID'])
    """SYNO END"""

    def handle_free(self, pyfile):
        """SYNO"""
        self.data = self.load(self.basic_download_free_url)

        self.solve_captcha()

        self.get_download_url()

        self.get_file_name(pyfile)
        """SYNO END"""

    def solve_captcha(self):
        """SYNO"""
        for _i in xrange(5):
            self.wait_and_retry_if_needed()

            action, inputs = self.parse_html_form("action='#'")
            if not inputs:
                self.error(_("Captcha form not found"))
            self.log_debug(inputs)

            m = re.search(self.CAPTCHA_PATTERN, self.data)
            if m is None:
                self.error(_("Captcha pattern not found"))
            captcha_url = m.group(1)
            inputs['captcha_response'] = self.captcha.decrypt(captcha_url)
            self.log_debug(inputs)

            self.data = self.load(self.basic_download_free_url, post=inputs)

            if '<div class="captcha-error">Incorrect, try again' in self.data:
                self.retry_captcha()
            else:
                self.captcha.correct()
                break
        else:
            self.fail(_("Invalid captcha"))
        """SYNO END"""

    """SYNO"""
    def wait_and_retry_if_needed(self):
        matched = re.search(self.LIMIT_WAIT_PATTERN, self.data)
        if matched is not None:
            wait_time = int(matched.group(1))
            self.wait(wait_time, True)
            self.retry()

    def get_download_url(self):
        self.wait_and_retry_if_needed()

        self.wait(63, False)

        self.req.http.c.setopt(pycurl.HTTPHEADER, ["X-Requested-With: XMLHttpRequest"])
        self.data = self.load("%s%s" % (self.LINK_DOWNLOAD_GET_LINK, self.info['pattern']['ID']))
        self.req.http.c.setopt(pycurl.HTTPHEADER, ["X-Requested-With:"])

        self.req.http.lastEffectiveURL = self.basic_download_free_url

        matched = re.search(self.LINK_FREE_PATTERN, self.data)
        if matched:
            self.link = self.LINK_DOMAIN + matched.group(1)
        else:
            self.retry()


    def get_file_name(self, pyfile):
        matched = re.search("%s/(.+)" % (self.info['pattern']['ID']), self.link)
        if matched:
            pyfile.name = matched.group(1)
    """SYNO END"""
