# -*- coding: utf-8 -*-

from module.plugins.internal.Plugin import Skip
from module.plugins.internal.misc import decode, parse_size

from module.plugins.internal.misc import (Config, DB, decode, encode, exists, fixurl, fsjoin,
                                          format_exc, html_unescape, parse_html_header, remove, set_cookies)
from module.network.RequestFactory import RequestFactory
from module.ConfigParser import ConfigParser


class FakeCore():
    def __init__(self):
        self.config = ConfigParser()

class Account():
    __name__ = "Account"
    __type__ = "account"
    __version__ = "0.84"
    __status__ = "stable"

    __description__ = """Base account plugin"""
    __license__ = "GPLv3"
    __authors__ = [("Walter Purcaro", "vuolter@gmail.com"),
                   ("Synology PAT", "pat@synology.com")]

    #: Relogin account every 30 minutes, use -1 for never expire, you have to explicitly call relogin() when needed
    LOGIN_TIMEOUT = 30 * 60
    TUNE_TIMEOUT = True     #: Automatically tune relogin interval


    def __init__(self, manager):

        self.manager = manager

        self.accounts = {}  #@TODO: Recheck in 0.4.10
        self.user     = None

        self.timeout = self.LOGIN_TIMEOUT

        self.requestFactory = RequestFactory(FakeCore())


    def premium(self, user, password, data):
        result = self.grab_info(user, password, data)
        return bool(result.get("premium", False))


    def _log(self, level, plugintype, pluginname, messages):
        pass


    def setup(self):
        """
        Setup for enviroment and other things, called before logging (possibly more than one time)
        """
        pass


    def periodical_task(self):
        raise NotImplementedError


    def signin(self, user, password, data):
        """
        Login into account, the cookies will be saved so user can be recognized
        """
        pass

    def login(self, username, password, data):
        bl_valid = False
        self.req = self.requestFactory.getRequest(self.__name__, self.user)
        self.setup()

        try:
            self.signin(username, password, data)

        except Skip, e:
            bl_valid = True

        except Exception, e:
            bl_valid = False

        else:
            bl_valid = True

        finally:
            return bool(bl_valid)



    def relogin(self):
        return self.login()


    def grab_info(self, user, password, data):
        """
        This should be overwritten in account plugin
        and retrieving account information for user

        :param user:
        :param req: `Request` instance
        :return:
        """
        pass


    def parse_traffic(self, size, unit=None):  #@NOTE: Returns kilobytes only in 0.4.9
        self.log_debug("Size: %s" % size,
                       "Unit: %s" % (unit or "N/D"))
        return parse_size(size, unit or "byte") / 1024  #@TODO: Remove `/ 1024` in 0.4.10


    def fail_login(self, msg="Login handshake has failed"):
        return self.fail(msg)


    def skip_login(self, msg="Already signed in"):
        return self.skip(msg)

    def log_error(self, *args, **kwargs):
        pass

    def log_warning(self, *args, **kwargs):
        pass

    def log_info(self, *args, **kwargs):
        pass

    def log_debug(self, *args, **kwargs):
        pass

    def fail(self, msg):
        raise Exception(str(msg))

    def load(self, url, get={}, post={}, ref=True, cookies=True, just_header=False, decode=True,
             multipart=False, redirect=True, req=None):
        """
        Load content at url and returns it

        :param url:
        :param get:
        :param post:
        :param ref:
        :param cookies:
        :param just_header: If True only the header will be retrieved and returned as dict
        :param decode: Wether to decode the output according to http header, should be True in most cases
        :return: Loaded content
        """
        """
        if self.pyload.debug:
            self.log_debug("LOAD URL " + url,
                           *["%s=%s" % (key, value) for key, value in locals().items()
                             if key not in ("self", "url", "_[1]")])
        """

        url = fixurl(url, unquote=True)  #: Recheck in 0.4.10

        if req is False:
            req = get_request()
            req.setOption("timeout", 60)  #@TODO: Remove in 0.4.10

        elif not req:
            req = self.req

        #@TODO: Move to network in 0.4.10
        if isinstance(cookies, list):
            set_cookies(req.cj, cookies)

        #@TODO: Move to network in 0.4.10
        if not redirect:
            req.http.c.setopt(pycurl.FOLLOWLOCATION, 0)

        elif type(redirect) is int:
            req.http.c.setopt(pycurl.MAXREDIRS, redirect)

        html = req.load(url, get, post, ref, bool(cookies), just_header, multipart, decode is True)  #@TODO: Fix network multipart in 0.4.10

        #@TODO: Move to network in 0.4.10
        if not redirect:
            req.http.c.setopt(pycurl.FOLLOWLOCATION, 1)

        elif type(redirect) is int:
            maxredirs = int(self.pyload.api.getConfigValue("UserAgentSwitcher", "maxredirs", "plugin")) or 5  #@TODO: Remove `int` in 0.4.10
            req.http.c.setopt(pycurl.MAXREDIRS, maxredirs)

        #@TODO: Move to network in 0.4.10
        if decode:
            html = html_unescape(html)

        #@TODO: Move to network in 0.4.10
        if isinstance(decode, basestring):
            html = decode(html, decode)

        self.last_html = html

        """
        if self.pyload.debug:
            self.dump_html()
        """

        #@TODO: Move to network in 0.4.10
        header = {'code': req.code, 'url': req.lastEffectiveURL}
        header.update(parse_html_header(req.http.header if hasattr(req, "http") else req.header))  #@NOTE: req can be a HTTPRequest or a Browser object

        self.last_header = header

        if just_header:
            return header
        else:
            return html
