# -*- coding: utf-8 -*-

from module.plugins.internal.SynoAccount import Account
from os import stat
from os.path import join
import time
from module.plugins.internal.misc import get_pattern_by_hoster_data


class MultiAccount(Account):
    __name__ = "MultiAccount"
    __type__ = "account"
    __version__ = "0.05"
    __status__ = "testing"

    __config__ = [("activated" , "bool", "Activated", True),
                  ("multi", "bool", "Multi-hoster", True),
                  ("multi_mode", "all;listed;unlisted", "Hosters to use", "all"),
                  ("multi_list", "str" , "Hoster list (comma separated)", ""),
                  ("multi_interval", "int", "Reload interval in hours", 12)]

    __description__ = """Multi-hoster account plugin"""
    __license__ = "GPLv3"
    __authors__ = [("Walter Purcaro", "vuolter@gmail.com"),
                   ("Synology PAT", "pat@synology.com")]

    def get_pattern(self, username, password):
        pattern = self.load_saved_pattern()
        if pattern:
            return pattern
        #need login before grab_hosters
        bl_valid = self.login(username, password, {})
        if not bl_valid:
            return None
        pattern = self.get_updated_pattern(username, password)
        self.save_pattern(pattern)
        return pattern

    def get_updated_pattern(self, username, password):
        hoster_data = self.grab_hosters(username, password, {})
        regexp = get_pattern_by_hoster_data(self, hoster_data)
        return regexp

    def get_pattern_filename(self):
        return join("/tmp", "synodl_" + self.__name__ + "_pattern")

    def save_pattern(self, pattern):
        filename = self.get_pattern_filename()
        try:
            with open(filename, "w") as write_file:
                write_file.write("%s" % pattern)
        except:
            pass

    def load_saved_pattern(self):
        filename = self.get_pattern_filename()
        try:
            mtime = stat(filename).st_mtime
            time_diff = time.time() - mtime
            if time_diff > 86400:
                return None;
            with open(filename, 'r') as content_file:
                content = content_file.read()
        except:
            return None
        return content;
